/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
#ifndef XMAP_H
#define XMAP_H

#include <X11/Intrinsic.h>

/* Should make this configurable */
#define COMMON_PATH "/usr/local/hugo"

#undef	DND

#define NUMOBJ 16	// Number of objects in pixmap index

#ifndef MIN
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
#endif
#ifndef ABS
#define ABS(a)          ((a > 0) ? (a) : 0 - (a))
#endif
#ifndef SIGN
#define SIGN(a)         ((a > 0) ? 1 : -1)
#endif

//#define ABSDIFF(a,b)    (((a) - (b)) < 0 ? (b) - (a) : (a) - (b))
//#define ABSDIFF(a,b)    (((a) - (b)) < 0 ? -((a) - (b)) : (a) - (b))
#define ABSDIFF(a,b)    (ABS(a-b))

#ifndef PI
#define PI        3.1415926535897931160E0      /* pi */
#endif
#define DEGREE	(PI/180.0)
#define RADIAN	(180.0/PI)

#define M_BSBDIR	0x0001		// Folder type
#define M_BSBFILE	0x0002		// File is BSB
#define M_BSBSHEET	0x0004		// Sheet in folder
#define M_SHAPE		0x0008		// Shape file

typedef struct {
    double lat, lon;
} Coord; 

typedef struct {
    double lat, lon;
    double alarm;
    char *name;
    char *description;
    char *action;
} WP; 

struct tbutton {
        int  active;
	int  spacing;
        char **data;
        char *tip;
        void (*func)();
        XtPointer func_data; /* client_data for func() */
        Pixmap pmap;
        Widget w;
};

struct features {
	int  active;
	char **data;
	Pixmap pmap;
};

struct objects {
	int  active;
	char **data;
	Pixmap pmap;
	Pixmap smap;
};

struct stamps {
	char **data;
	Pixmap pmap;
	Pixmap smap;
	char xhot;
	char yhot;
};

struct map {
	char *desname;
	char *description;
	char *filename;
	char *ll;
	char *ref1;
	char *ref2;
	char *waw;
        char *nmap; //adjacent maps
        char *nemap;
        char *emap;
        char *semap;
        char *smap;
        char *swmap;
        char *wmap;
        char *nwmap;
        double nedge;
        double eedge;
        double sedge;
        double wedge;
	Pixmap pmap;
	Dimension mwidth, mheight;
};

struct mapdata {
	struct map *map;
        double pixd_x;
        double pixd_y;

        double nulx; //longitude of left edge
        double nuly; //latitude of top edge

	double clat, clon;
	double wlat, wlon;

        int mx1, my1;
        int mx2, my2;
        double lat1, lon1;
        double lat2, lon2;

        short mapdatum;
	short type;
        short calibrated;
        short calstate;
        short mx_offset;
        short my_offset;
	short zoom;
};

struct DATUM {
        char *name;
        short ellipsoid;
        double dx;
        double dy;
        double dz;
};

struct ELLIPSOID {
        char *name;             // name of ellipsoid
        double a;               // semi-major axis, meters
        double invf;            // 1/f
};

struct wp_private {
	char *name;
	char *desc;
};

struct db_entry {
        char *description;
        char *name;
        int flags;
	struct db_entry *next, **prev_p;
};

enum {SRASTER,DRG,BSB,VECTOR,SHAPE};

enum {ANS_NONE, ANS_OK, ANS_CANCEL};
enum { UNKNOWN, ENGLISH, METRIC, NAUTICAL };
extern int unit;
extern char *unit_names[];

extern struct tbutton tbutton[];
extern struct tbutton hbutton[];
extern struct features pbutton[];
extern struct features features[];
extern struct stamps objects[];
extern struct map map;
extern struct mapdata mapdata;
extern char *inifile;
extern char *filename;
extern char *on_off[];

extern char buf[];	/* for general use (256 bytes) */

extern Widget GetTopShell(Widget w);
extern Widget showHelpToolBarButton(int button, int ord, Pixmap * pmap, void *p, XtPointer p_data);
extern Pixel get_pixel(Widget w, char *color);
extern void update_monitor_units(void);

extern void save_geometry(Widget w, char *name);
extern void save_position(Widget w, char *name);

extern Pixmap get_mark_pixmap(int obj);
extern Pixmap get_mark_mask(int obj);

extern Pixmap get_darrow_pixmap(int obj);

extern GC status1GC;
extern GC status2GC;
extern GC status3GC;

struct GeoRef {
  int status;
  int count;
  int order;
  double *e1;
  double *n1;
  double *n2;
  double *e2;
};

/* BEGIN maps.c */
extern void import_map();
/* END maps.c */

/* BEGIN drawobject.c */
extern void import_markers();
/* END drawobject.c */

/* BEGIN scale.c */
double cv_dist(double dist);

#define KM2NMILES 1.852
#define KM2MILES  1.609347
/* END scale.c */
/* BEGIN zoom.c */
void zoomin(Widget w, struct mapdata *mapdata);
void zoomout(Widget w, struct mapdata *mapdata);
void zoomout4(Widget w, struct mapdata *mapdata);

/* END zoom.c */
/* BEGIN make_menu.c */
typedef struct _menu_item {
  char        *label;         /* the label for the item */
  WidgetClass *class;         /* pushbutton, label, separator... */
  char         mnemonic;      /* mnemonic; NULL if none */
  char        *accelerator;   /* accelerator; NULL if none */
  char        *accel_text;    /* to be converted to compound string */
  void       (*callback)();   /* routine to call; NULL if none */
  XtPointer    callback_data; /* client_data for callback() */
  struct _menu_item *subitems;/* pullright menu items, if not NULL */
  Widget              widget; /* the widget for this item */
} MenuItem;

extern Widget BuildMenu(Widget parent, int menu_type, char *menu_title,
                                char menu_mnemonic, MenuItem *items);
extern Widget BuildPulldownMenu(Widget parent, char *menu_title,
                                char menu_mnemonic, Boolean tear_off,
                                MenuItem *items);
/* END make_menu.c */
/* BEGIN tip.c */
typedef struct {
    Boolean quiet;
    Boolean menubar;
    Boolean toolbar;
    Boolean statusbar;
    Boolean grids;
    Boolean toolhelp;
    Boolean instruments;
    Boolean speedometer;
    Boolean aspeedometer;
    Boolean position;
    Boolean utmpos;
    Boolean nicecol;
    Boolean useutc;
    Boolean pdebug;
    String mapfile;
    int format; /* enum FORMAT */
} OptionsRec;

extern OptionsRec options;

enum FORMAT {DMS, DMM, DDD, UTM};

void do_tip(Widget main_w, Widget w, char *tip, Position offset, OptionsRec *options);
void undo_tip(OptionsRec *options);
/* END tip.c */
/* BEGIN datum.c */
void molod_translate(short fromWGS84, double *latitude, double *longitude, 
		     double dx, double dy, double dz, double a, double f);
/* END datum.c */
/* BEGIN conv.c */
void datumParams(short datum, double *a, double *es);

void toUserFormat(int format, double a, char *bufp, int bufplen);
void toUserFormatLat(int format, double a, char *bufp, int bufplen);
void toUserFormatLon(int format, double a, char *bufp, int bufplen);

double fromUserFormat(int format, char *a);

void toRawLatDMM(double a, char *bufp, int bufplen);
void toRawLonDMM(double a, char *bufp, int bufplen);

void DegToUTM(double lat, double lon, char *zone, double *x, double *y);
void UTMtoDeg(short zone, short southernHemisphere, double x, double y, double *lat, double *lon);
/* END conv.c */

extern int georef(double e1, double n1, double *e, double *n, double E[], 
		  double N[], int order);
extern int compute_georef_equations(struct GeoRef *cp, double E12[], 
				    double N12[], double E21[], double N21[]);

extern double getlat(void);
extern double getlon(void);
extern double y2lat(int);
extern double x2lon(int);
extern double distance(double lata, double lona, double latb, double lonb);
extern double heading(double lata, double lona, double latb, double lonb);

extern void latlontoxy(double latitude, double longitude, Position *x, Position *y);
extern void xytolatlon(Position x, Position y, double *lat, double *lon);

extern void show_help_cb(Widget dialog, XtPointer client_data, XtPointer call_data);

extern int route_flush(int save);

extern char *mystrsep( char **stringp, char *delim);
extern char *mystrsep2( char **stringp, char *delim);

extern char *bsb_getval_kap(char *rec, char *fld, char *dest, int len);

extern void set_cursor(int busy);
extern int mode_is(int qmode);
extern void mark_move(int x, int y);
extern int is_box_on(void);
extern void draw_box(Widget widget, int sx, int sy, int ex, int ey);
extern void undraw_box(Widget w);
extern void draw_rubber(Widget widget, int sx, int sy, int ex, int ey);
extern void undraw_rubber(Widget widget);
extern int lldialog(char *title, double lat, double lon);
extern void mark(int x, int y);
extern void change_vector(int x1, int y1, int x2, int y2);
extern void change_raster(int x1, int y1, int x2, int y2);
extern void redraw_mark(Widget w, void *client_data, void *cbs);
extern void redraw_objects(void);
extern void redraw_location(void);
extern void route(int x, int y);
extern void beep(void);
extern int route_change(int sx, int sy, int ex, int ey);
extern int place_object(int x, int y);
extern void route_translate_rubber(int x, int y);
extern void center(int x, int y);
extern void vcenter(int x, int y);
extern void draw_scale(void);
extern void project(double lat, double lon, Position * x, Position * y);
extern void bsb_project(double lat, double lon, Position * x, Position * y);
extern void gtif_project(double lat, double lon, Position * x, Position * y);
extern void decode(Position x, Position y, double *lat, double *lon);
extern void bsb_decode(Position x, Position y, double *lat, double *lon);
extern void gtif_decode(Position x, Position y, double *lat, double *lon);
extern void load_map(char *map);
extern void init_map(struct map *map);
extern void vector_map(struct map *map);
extern void error(char *errmsg, int Errno);
extern void mark_track(void);
extern void unmark_track(void);
extern void show_log_status(void);
extern void navigate_next(void);
extern void vzoomout4(void);
extern void dzoomout4(void);
extern void bzoomout4(void);
extern void vzoomin(void), vzoomout(void);
extern void dzoomin(void), dzoomout(void);
extern void bzoomin(void), bzoomout(void);
extern int do_calibrate(void);
extern void write_des(struct map *map);
extern void select_marker_file(Widget w);
extern void draw_route(Widget widget, XPoint * p, int n, int flag);
extern void draw_object_pixmap(int index, int x, int y);
extern void highlight(Widget widget, XPoint * p, int n);
extern void refresh(int warp);
extern Status myXAllocColor(register Display * dpy, Colormap cmap, XColor * xcolor);
extern int GetPrivateProfileString(char *section, char *entry, char *def_val,
                            char *buffer, int len, char *filename);
extern unsigned GetPrivateProfileInt(char *section, char *entry, int def_val,
                              char *filename);
extern double GetPrivateProfileFloat(char *section, char *entry, 
				     double def_val, char *filename);
int WritePrivateProfileString(char *section, char *entry, char *string,
                              char *filename);
extern void register_canvas(Widget w);
extern void instrument_init(void);
extern void create_help_dialog(void);
extern void error_wait_ack(void);
extern void error_wait(char *errmsg, int Errno);
extern void strtolower(char *p);
extern void do_help(void);
extern void ch_title(char *s);

extern struct db_entry *get_db_entry(void);
extern void kill_db_entry(struct db_entry *);

extern char *get_mapdir();
extern char *get_vmap_root();

/* These macros are for insertion into and deletion from a singly-linked list
 * with back pointers to the previous element's next pointer.  In order to
 * make these macros act like expressions, they use the comma operator for
 * sequenced evaluations of assignment, and "a && b" for "evaluate assignment
 * b if expression a is true". */
#define LIST_INSERT(head, elem) \
        ((elem)->next = *(head), \
         (*head) && ((*(head))->prev_p = &(elem)->next), \
         (*head) = (elem), (elem)->prev_p = (head))
#define LIST_INSERT_TAIL(head, tail, elem) \
        ((elem)->prev_p = NULL, \
         (*tail) && ((*(tail))->next = (elem), \
         (elem)->prev_p = &(*(tail))->next), \
         (*tail) = (elem), \
         (!(*head)) && ((*head) = (elem), (elem)->prev_p = (head)), \
         (elem)->next = 0)
#define LIST_DELETE(elem) \
        (*(elem)->prev_p = (elem)->next, \
         (elem)->next && ((elem)->next->prev_p = (elem)->prev_p))

#endif /* XMAP_H */
